/**********************************************************************/
/* The 1st IPSJ SIG-ARC High-Performance Processor Design Contest     */
/* psend: program send and receive data            Version 2013-10-15 */
/**********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/time.h>

/**********************************************************************/
#define DATA_SIZE (512*1024) // data transfer size (default 512KB)
#define BAUD_RATE (B1000000) // serial baud rate  (default 1Mbaud)

/** function to return the current time                              **/
/**********************************************************************/
long long get_time(){
    struct timeval  tp;
    struct timezone tz;
    gettimeofday(&tp, &tz);
    return tp.tv_sec * 1000000ull + tp.tv_usec;
}

/**********************************************************************/
int main(int argc,char *argv[])
{
    int serial;
    struct termios tty;

    char buf[DATA_SIZE];
    long long time1, time2, time3;

    printf("# Simple Program Loader psend Version 0.2 (2013-10-15)\n");

    if(argc!=3){
        printf("usage : psend tty_device_name program_name\n");
        exit(1);
    }
    
    /* open a program file and read the image to buf[]                */
    /******************************************************************/
    {
        FILE *f;
        f = fopen(argv[2], "rb");
        if(f==NULL){
            printf("program_name %s is invalid.\n", argv[2]);
            exit(1);
        }
        
        fread(&buf, DATA_SIZE, 1, f);
        fclose(f);
        printf("# file read finished and file transferring...\n");
    }
    
    /* open the serial device                                         */
    /******************************************************************/
    serial = open(argv[1], O_RDWR | O_NOCTTY);
    if(serial==-1){
        fprintf(stdout,"can't open device %s\n", argv[1]);
        exit(1);
    }
    
    memset(&tty, 0, sizeof(tty));
    tty.c_cflag     = CS8 | CLOCAL | CREAD;
    tty.c_cc[VMIN]  = 1;
    tty.c_cc[VTIME] = 0;
    
    cfsetospeed(&tty, BAUD_RATE);
    cfsetispeed(&tty, BAUD_RATE);
    
    tcflush(serial, TCIFLUSH);
    tcsetattr(serial, TCSANOW, &tty);
    
    /* send the DATA_SIZE byte data to FPGA                           */
    /******************************************************************/
    time1 = get_time();

    int i;
    printf("# "); fflush(stdout);
    for(i=0; i<32; i++){
        write(serial, &buf[DATA_SIZE/32*i], DATA_SIZE/32);
        printf("."); fflush(stdout);
    }
    printf("\n# file transfer finished.\n");
    
    time2 = get_time();

    /* run the program and wait the output data                       */
    /******************************************************************/
    {
        char c1 = 0;
        char c2 = 0;
        char c3 = 0;

        while(1){
            c3 = c2;
            c2 = c1;
            read(serial, &c1, 1);
            printf("%c",c1);
            fflush(stdout);
            
            if(c3=='E' && c2=='N' && c1=='D') break;
        }
    }
    
    time3 = get_time();

    printf("\n");
    printf("# data transfer time:%9.3f\n", (time2 - time1)/1000000.0);
    printf("# execution     time:%9.3f\n", (time3 - time1)/1000000.0);

    close(serial);
    return 0;
}
/**********************************************************************/
